#!/bin/sh
###############################################################################
# IBM Confidential
# OCO Source Materials
# © Copyright IBM Corporation 217
# The source code for this program is not published or otherwise divested
# of its trade secrets, irrespective of what has been deposited with
# the U.S. Copyright office.
###############################################################################
error() {
    cd ${SPE_INSTALL_DIR}
    echo
    echo $1
    echo
    rm -rf ${WORK_DIR}
    exit 1
}

syntax() {
    echo "Deploys ${PRODUCT} Java patch resources"
    echo
    echo "speinstall-java-patch.sh -j /path/to/patch.zip"
    echo
    echo "Command line arguments:"
    echo "-j  (Required) Path to the ${PRODUCT} Java patch archive file"
    echo
}

if [ -z "$InSPEEnv" ]; then
    echo Please initialize the SPE environment before running this command.
    echo You can use spesetup.sh to initialize the environment.
else
    TIMESTAMP="D"`date +%Y%m%d`".T"`date +%H%M%S`
    BACKUP_DIR=backups/${TIMESTAMP}_JAVA_PATCH
    
    PRODUCT="SPE"
    SPE_V_R="2.0"
    localOS=`uname -s`
    localPROC=
    PATCH_ZIP=
    EXPECTED_ZIP=
    NO_ZIP_NAME=

    DIR_bin=d3adb33f
    DIR_lib=d3adb33f
    DIR_man=d3adb33f
    DIR_plugin=d3adb33f
    DIR_properties=d3adb33f
    FILE_COPYRIGHT=d3adb33f
    FILE_copyright=d3adb33f
    FILE_java_os11=d3adb33f
    FILE_jre_os11=d3adb33f
    FILE_notices=d3adb33f
    FILE_README=d3adb33f
    FILE_READMEhtml=d3adb33f
    FILE_Welcome=d3adb33f

    case ${localOS} in
        AIX) EXPECTED_ZIP=jre764redist.tar.gz
            NO_ZIP_NAME=jre764redist 
            DIR_bin=jre/bin
            DIR_lib=jre/lib
            DIR_man=jre/plugin
            DIR_properties=properties
            FILE_copyright=docs/content/en/copyright
            FILE_notices=notices.txt
            ;;
        HP-UX) EXPECTED_ZIP=hpia6470sr10fp1hybrid-jre.tar.Z
            NO_ZIP_NAME=hpia6470sr10fp1hybrid-jre 
            DIR_bin=bin
            DIR_lib=lib
            DIR_man=man
            DIR_properties=${SPE_INSTALL_DIR}/${BACKUP_DIR}/java/properties
            FILE_COPYRIGHT=COPYRIGHT
            FILE_copyright=copyright
            FILE_java_os11=java.os11.release.notes.html
            FILE_jre_os11=jre.os11.release.notes
            FILE_notices=notices.txt
            FILE_README=README
            ;;
        Linux)
            localPROC=`uname -p`
            case ${localPROC} in
                ppc64) EXPECTED_ZIP=
                    error "Linux ppc64 not supported in SPE 2.0/8.5"
                    ;;
                s390x) EXPECTED_ZIP=ibm-java-jre-7.0-10.1-linux-s390x.tgz
                    NO_ZIP_NAME=ibm-java-jre-7.0-10.1-linux-s390x
                    DIR_bin=ibm-java-s390x-70/jre/bin
                    DIR_lib=ibm-java-s390x-70/jre/lib
                    DIR_properties=ibm-java-s390x-70/properties
                    FILE_copyright=ibm-java-s390x-70/copyright
                    FILE_notices=ibm-java-s390x-70/notices.txt
                    ;;
                x86_64) EXPECTED_ZIP=ibm-java-jre-7.0-10.1-linux-x86_64.tgz
                    NO_ZIP_NAME=ibm-java-jre-7.0-10.1-linux-x86_64
                    DIR_bin=ibm-java-x86_64-70/jre/bin
                    DIR_lib=ibm-java-x86_64-70/jre/lib
                    DIR_plugin=ibm-java-x86_64-70/jre/plugin
                    DIR_properties=ibm-java-x86_64-70/properties
                    FILE_copyright=ibm-java-x86_64-70/copyright
                    FILE_notices=ibm-java-x86_64-70/notices.txt
                    ;;
                *) error "${localOS} ${localPROC} is not supported"
                    ;;
            esac        
            ;;
        SunOS) EXPECTED_ZIP=sol6470sr10fp1hybrid-jre.tar.Z
            NO_ZIP_NAME=sol6470sr10fp1hybrid-jre
            DIR_bin=bin
            DIR_lib=lib
            DIR_man=plugin
            DIR_properties=${SPE_INSTALL_DIR}/${BACKUP_DIR}/java/properties
            FILE_COPYRIGHT=COPYRIGHT
            FILE_copyright=copyright
            FILE_notices=notices.txt
            FILE_README=README
            FILE_READMEhtml=${SPE_INSTALL_DIR}/${BACKUP_DIR}/java/README.html
            FILE_Welcome=Welcome.html
            ;;
        *) error "${localOS} ${localPROC} is not supported"
            ;;
    esac
    
    # Get input
    while getopts j: option
    do
        case ${option} in
            j) PATCH_ZIP=${OPTARG}
               ;;
           \?) syntax
               exit
               ;;
        esac
    done

    # Validate the PATCH_ZIP
    echo PATCH_ZIP=${PATCH_ZIP}
    if [ -z "${PATCH_ZIP}" ]
    then
        echo "Missing mandatory parameter -j"
        syntax
        exit 1
    else
        if [ ! -f "${PATCH_ZIP}" ]
        then
            if [ "${PATCH_ZIP}" != "nozip" ]
            then
                error "SPE Java patch archive file does not exist: ${PATCH_ZIP}"
            fi
        else
            if [[ "${PATCH_ZIP}" != *"${EXPECTED_ZIP}" ]]
            then
                echo
                echo "Expected file for ${localOS} ${localPROC}: ${EXPECTED_ZIP}"
                error "Invalid Java archive file name: ${PATCH_ZIP}"
            fi
        fi
    fi

    ZIP_NAME=`basename "${PATCH_ZIP}"`
    echo ZIP_NAME=${ZIP_NAME}
    
    # Validate SPE Version
    if [ ! -f "${SPE_INSTALL_DIR}/jars/spe-${SPE_V_R}-core.jar" ]    
    then
        error "${ZIP_NAME} cannot be applied to ${PRODUCT} ${SPE_V_R}"
    fi

    # Generate WORK_DIR name and extract zip to it
    echo NO_ZIP_NAME=${NO_ZIP_NAME}
    WORK_DIR="${SPE_INSTALL_DIR}/${NO_ZIP_NAME}"
    echo WORK_DIR=${WORK_DIR}
    if [ -d ${WORK_DIR} ]
    then
        rm -rf ${WORK_DIR}
    fi
    mkdir ${WORK_DIR}
    cd ${WORK_DIR}
    if [ $? -ne 0 ]
    then
        error "Failed to change directory to ${WORK_DIR}"
    fi
    cp -p ${PATCH_ZIP} .

    echo
    echo "Extracting files..."
    UCMD=""
    for UZCMD in gunzip uncompress compress
    do
        UCMD=$(command -v ${UZCMD} 2> /dev/null)
        if [ $? -eq 0 ]
        then
            break;
        fi
    done

    echo "UCMD=${UCMD}"
    if [ "${UCMD}" = "" ]
    then
        error "Failed to find an uncompress program"
    fi

    if [ ! -x ${UCMD} ]
    then
        error "${UCMD} command is not executable"
    fi

    ${UCMD} ${ZIP_NAME}
    if [ $? -ne 0 ]
    then
        error "Failed to uncompress ${ZIP_NAME} file"
    fi               

    TFILE=${NO_ZIP_NAME}.tar
    echo "TFILE=${TFILE}"
    tar -xvf ${TFILE}
    if [ $? -ne 0 ]
    then
        error "Failed to untar ${TFILE} file"
    fi

    rm -rf ${TFILE}
    if [ $? -ne 0 ]
    then
        error "Failed to remove ${TFILE} file"
    else
        echo "Successfully extraction of the ${TFILE} file" 
    fi

    # Create backup directory
    cd "${SPE_INSTALL_DIR}"
    if [ ! -d BACKUP_DIR ]
    then
        echo
        echo "Creating ${BACKUP_DIR} directory"
        mkdir -p ${BACKUP_DIR}
        if [ $? -ne 0 ]
        then
            error "Failed to create ${BACKUP_DIR} directory"
        fi
    fi
    
    if [ -d java ]
    then
        echo
        echo "Backing up ${SPE_INSTALL_DIR}/java..."
        mv java ${BACKUP_DIR}/.
        if [ $? -ne 0 ]
        then
            error "Failed to move ${SPE_INSTALL_DIR}/java to ${BACKUP_DIR}"
        fi
    fi

    # Deploy patch files
    echo
    echo "Deploying patch files..."
    if [ ! -d java ]
    then
        echo
        mkdir java
        if [ $? -ne 0 ]
        then
            error "Failed to create ${SPE_INSTALL_DIR}/java directory"
        fi
    fi

    cd ${WORK_DIR}

    if [ -d ${DIR_bin} ]
    then
        echo "Moving ${DIR_bin} to ${SPE_INSTALL_DIR}/java"
        mv ${DIR_bin} ../java/.
        if [ $? -ne 0 ]
        then
            error "Failed to move ${DIR_bin} to ${SPE_INSTALL_DIR}/java"
        fi
    fi

    if [ -d ${DIR_lib} ]
    then
        echo "Moving ${DIR_lib} to ${SPE_INSTALL_DIR}/java"
        mv ${DIR_lib} ../java/.
        if [ $? -ne 0 ]
        then
            error "Failed to move ${DIR_lib} to ${SPE_INSTALL_DIR}/java"
        fi
    fi

    if [ -d ${DIR_man} ]
    then
        echo "Moving ${DIR_man} to ${SPE_INSTALL_DIR}/java"
        mv ${DIR_man} ../java/.
        if [ $? -ne 0 ]
        then
            error "Failed to move ${DIR_man} to ${SPE_INSTALL_DIR}/java"
        fi
    fi

    if [ -d ${DIR_plugin} ]
    then
        echo "Moving ${DIR_plugin} to ${SPE_INSTALL_DIR}/java"
        mv ${DIR_plugin} ../java/.
        if [ $? -ne 0 ]
        then
            error "Failed to move ${DIR_plugin} to ${SPE_INSTALL_DIR}/java"
        fi
    fi

    if [ -d ${DIR_properties} ]
    then
        echo "Moving ${DIR_properties} to ${SPE_INSTALL_DIR}/java"
        mv ${DIR_properties} ../java/.
        if [ $? -ne 0 ]
        then
            error "Failed to move ${DIR_properties} to ${SPE_INSTALL_DIR}/java"
        fi
    fi

    if [ -f ${FILE_COPYRIGHT} ]
    then
        echo "Moving ${FILE_COPYRIGHT} to ${SPE_INSTALL_DIR}/java"
        mv ${FILE_COPYRIGHT} ../java/.
        if [ $? -ne 0 ]
        then
            error "Failed to move ${FILE_COPYRIGHT} to ${SPE_INSTALL_DIR}/java"
        fi
    fi

    if [ -f ${FILE_copyright} ]
    then
        echo "Moving ${FILE_copyright} to ${SPE_INSTALL_DIR}/java"
        mv ${FILE_copyright} ../java/.
        if [ $? -ne 0 ]
        then
            error "Failed to move ${FILE_copyright} to ${SPE_INSTALL_DIR}/java"
        fi
    fi

    if [ -f ${FILE_java_os11} ]
    then
        echo "Moving ${FILE_java_os11} to ${SPE_INSTALL_DIR}/java"
        mv ${FILE_java_os11} ../java/.
        if [ $? -ne 0 ]
        then
            error "Failed to move ${FILE_java_os11} to ${SPE_INSTALL_DIR}/java"
        fi
    fi

    if [ -f ${FILE_jre_os11} ]
    then
        echo "Moving ${FILE_jre_os11} to ${SPE_INSTALL_DIR}/java"
        mv ${FILE_jre_os11} ../java/.
        if [ $? -ne 0 ]
        then
            error "Failed to move ${FILE_jre_os11} to ${SPE_INSTALL_DIR}/java"
        fi
    fi

    if [ -f ${FILE_notices} ]
    then
        echo "Moving ${FILE_notices} to ${SPE_INSTALL_DIR}/java"
        mv ${FILE_notices} ../java/.
        if [ $? -ne 0 ]
        then
            error "Failed to move ${FILE_notices} to ${SPE_INSTALL_DIR}/java"
        fi
    fi

    if [ -f ${FILE_README} ]
    then
        echo "Moving ${FILE_README} to ${SPE_INSTALL_DIR}/java"
        mv ${FILE_README} ../java/.
        if [ $? -ne 0 ]
        then
            error "Failed to move ${FILE_README} to ${SPE_INSTALL_DIR}/java"
        fi
    fi

    if [ -f ${FILE_READMEhtml} ]
    then
        echo "Moving ${FILE_READMEhtml} to ${SPE_INSTALL_DIR}/java"
        mv ${FILE_READMEhtml} ../java/.
        if [ $? -ne 0 ]
        then
            error "Failed to move ${FILE_READMEhtml} to ${SPE_INSTALL_DIR}/java"
        fi
    fi

    if [ -f ${FILE_Welcome} ]
    then
        echo "Moving ${FILE_Welcome} to ${SPE_INSTALL_DIR}/java"
        mv ${FILE_Welcome} ../java/.
        if [ $? -ne 0 ]
        then
            error "Failed to move ${FILE_Welcome} to ${SPE_INSTALL_DIR}/java"
        fi
    fi

    # Cleanup
    cd ${SPE_INSTALL_DIR}
    rm -rf ${WORK_DIR}
    echo
    echo "${PRODUCT} Java patch resources were deployed successfully"
    echo
fi
